package com.gic.finance.util.cashwithdrawal.event;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.AsyncCallbackUtils;
import com.gic.finance.constant.WithdrawalStatusEnum;
import com.gic.finance.dto.CashWithdrawalDTO;
import com.gic.finance.entity.TabCashWithdrawal;
import com.gic.finance.service.CashWithdrawalService;
import com.gic.finance.util.cashwithdrawal.AbstractState;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 审核
 * @ClassName: CheckAuthState 
 * @Description:  
 * @author guojuxing 
 * @date 2020/11/6 3:28 PM 
 */
@Component
public class CheckAuthState extends AbstractState {
    @Autowired
    private CashWithdrawalService cashWithdrawalService;

    @Override
    public ServiceResponse<String> cashWithdrawal(CashWithdrawalDTO dto) {
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "不能重复提审");
    }

    @Override
    public ServiceResponse<Void> approval(TabCashWithdrawal record) {
        record.setCashWithdrawalStatus(WithdrawalStatusEnum.PASS.getCode());
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> reject(TabCashWithdrawal record) {
        if (StringUtils.isBlank(record.getRejectReason())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "驳回理由不能为空");
        }
        record.setCashWithdrawalStatus(WithdrawalStatusEnum.REJECT.getCode());
        cashWithdrawalService.update(record);
        //回调
        AsyncCallbackUtils.callBack(record, "com.gic.open.api.service.market.PayCallbackApiService", "callbackWithdraw");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancel(TabCashWithdrawal record) {
        record.setCashWithdrawalStatus(WithdrawalStatusEnum.CANCEL.getCode());
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> complete(TabCashWithdrawal record) {
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "审核中不能打款");
    }
}
